==> libggi_1.99.2.0b2.1.orig.tar.gz: libggi-2.0b2.1/display/fbdev/pan.c <==
/* $Id: pan.c,v 1.22 1999/03/01 20:18:28 marcus Exp $
******************************************************************************

   Display-FBDEV

   Copyright (C) 1998 Andrew Apted    [andrew@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <linux/fb.h>

#include <ggi/internal/ggi-dl.h>
#include <ggi/display/fbdev.h>


int GGI_fbdev_setorigin(ggi_visual *vis, int x, int y)
{
	fbdev_hook *ff = LIBGGI_PRIVATE(vis);
	int max_x = LIBGGI_VIRTX(vis) - LIBGGI_X(vis);
	int max_y = LIBGGI_VIRTY(vis) - LIBGGI_Y(vis);
	int err=0;

	if ((ff->fix.xpanstep == 0) && (ff->fix.ypanstep == 0)) {
		GGIDPRINT("display-fbdev: panning not supported.\n");
		return -1;
	}
	
	if ((x < 0) || (y < 0) || (x > max_x) || (y > max_y)) {
		GGIDPRINT("display-fbdev: panning out of range:"
			"(%d,%d) > (%d,%d)\n", x, y, max_x, max_y);
		return -1;
	}

	if (ff->fix.xpanstep == 0) {
		x = 0;
	}
	if (ff->fix.ypanstep == 0) {
		y = 0;
	}

	ff->var.xoffset = x;
	ff->var.yoffset = y + vis->d_frame_num * LIBGGI_VIRTY(vis);

	err = ioctl(LIBGGI_FD(vis), FBIOPAN_DISPLAY, &ff->var); 
	
	if (err) {
		GGIDPRINT("display-fbdev: PAN_DISPLAY failed.\n");
		return err;
	}

	vis->origin_x = x;
	vis->origin_y = y;
	
	return 0;
}

int GGI_fbdev_setdisplayframe(ggi_visual *vis, int num)
{
        ggi_directbuffer *db = _ggi_db_find_frame(vis, num);

        if (db == NULL) {
                return -1;
        }
        vis->d_frame_num = num;

	return GGI_fbdev_setorigin(vis, vis->origin_x, vis->origin_y);
}

==> libggi-2.0b1.tar.gz: libggi-2.0b1/display/fbdev/pan.c <==
/* $Id: pan.c,v 1.19 1998/12/20 21:30:55 marcus Exp $
******************************************************************************

   Display-FBDEV

   Copyright (C) 1998 Andrew Apted    [andrew@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <linux/fb.h>

#include <ggi/internal/ggi-dl.h>
#include <ggi/display/fbdev.h>

void _GGI_fbdev_save_panning(ggi_visual *vis)
{
}

void _GGI_fbdev_restore_panning(ggi_visual *vis)
{
	fbdev_hook *ff = LIBGGI_PRIVATE(vis);

	if ((ff->var.xoffset != 0) && (ff->var.yoffset != 0)) {
		ioctl(LIBGGI_FD(vis), FBIOPAN_DISPLAY, 
			&FBDEV_PRIV(vis)->orig_var);
	}
}


/* ---------------------------------------------------------------------- */


int GGI_fbdev_setorigin(ggi_visual *vis, int x, int y)
{
	fbdev_hook *ff = LIBGGI_PRIVATE(vis);

	int max_x = LIBGGI_VIRTX(vis) - LIBGGI_X(vis);
	int max_y = LIBGGI_VIRTY(vis) - LIBGGI_Y(vis);

	int err=0;

	if ((ff->fix.xpanstep == 0) && (ff->fix.ypanstep == 0) &&
	    (ff->fix.ywrapstep == 0)) {
		DPRINT("display-fbdev: panning not supported.\n");
		return -1;
	}
	
	if ((x < 0) || (y < 0) || (x > max_x) || (y > max_y)) {
		DPRINT("display-fbdev: panning out of range:"
			"(%d,%d) > (%d,%d)\n", x, y, max_x, max_y);
		return -1;
	}

	if (ff->fix.xpanstep == 0) {
		x = 0;
	}
	if ((ff->fix.ypanstep == 0) || (ff->fix.ywrapstep == 0)) {
		y = 0;
	}

	ff->var.xoffset = x;
	ff->var.yoffset = y + vis->d_frame_num * LIBGGI_VIRTY(vis);

	err = ioctl(LIBGGI_FD(vis), FBIOPAN_DISPLAY, &ff->var); 
	
	if (err) {
		DPRINT("display-fbdev: PAN_DISPLAY failed.\n");
		return err;
	}

	vis->origin_x = x;
	vis->origin_y = y;
	
	return 0;
}

int GGI_fbdev_setdisplayframe(ggi_visual *vis, int num)
{
        ggi_directbuffer *db = _ggi_db_find_frame(vis, num);

        if (db == NULL) {
                return -1;
        }
        vis->d_frame_num = num;

	return GGI_fbdev_setorigin(vis, vis->origin_x, vis->origin_y);
}
